function approx_sol = approx_ode(f,a,b,initial_val,N,test_case)
%function approx_sol = approx_ode(f,a,b,initial_val,test_case)
%calculates the approximate solution for y'=f(t,y(t)) y(a) = initial_val
%the number of time steps is N
%the variable test_case allows for multiple timestepping schemes to be
%added later on
% A - Euler's method.
% B - Second order Taylor method.
%PC MA428 10/25/2015

approx_sol(1) = initial_val;
h=(b-a)/N; % size of time step
switch(test_case)
    case('A')
        for i = 2:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            approx_sol(i) = approx_sol(i-1) + h*feval(f,ti_minus1,approx_sol(i-1));
        end
    case('B')%second order Taylor method
        for i = 2:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            approx_sol(i) = approx_sol(i-1) + h*feval(f,ti_minus1,approx_sol(i-1))+ ((h^2)/2)*feval('yprime',ti_minus1,approx_sol(i-1)) ;
        end
    case('C')%fouth order Taylor
        for i=2:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            approx_sol(i) = approx_sol(i-1)+ h*feval(f,ti_minus1,approx_sol(i-1))+((h^2)/2)*feval('yprime',ti_minus1,approx_sol(i-1))+((h^3)/6)*feval('ydprime',ti_minus1,approx_sol(i-1))+ ((h^4)/24)*feval('ytprime',ti_minus1,approx_sol(i-1));
        end
    case('D') %RK2
         for i=2:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            w_tilde = approx_sol(i-1) + (h/2)*feval(f,ti_minus1,approx_sol(i-1));
            approx_sol(i) = approx_sol(i-1) + h*feval(f,(ti_minus1 + 0.5*h),approx_sol(i-1));
         end
    case('E')%RK4
        for i=2:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            k1 =  h*feval(f,ti_minus1,approx_sol(i-1));
            k2 =  h*feval(f,(ti_minus1+0.5*h),(approx_sol(i-1) +0.5*k1));
            k3 =  h*feval(f,(ti_minus1+0.5*h),(approx_sol(i-1) + 0.5*k2));
            k4 =  h*feval(f,(ti_minus1 + h),(approx_sol(i-1)  + k3));
            approx_sol(i) = approx_sol(i-1) + (1/6)*(k1+2*k2+2*k3+k4);
        end
    case('F') %Second order multistep method
        %get w2 from the second order Taylor method
        i=2;
        ti = a+(i-1)*h;
        ti_minus1 = a+(i-2)*h;
        f_val_timinus1 =feval(f,ti_minus1,approx_sol(i-1));
        approx_sol(i) = approx_sol(i-1) + h*feval(f,ti_minus1,approx_sol(i-1))+ ((h^2)/2)*feval('yprime',ti_minus1,approx_sol(i-1)) ;
        
        for i=3:(N+1)
            ti = a+(i-1)*h;
            ti_minus1 = a+(i-2)*h;
            f_val_ti= feval(f,ti_minus1,approx_sol(i-1)); 
            approx_sol(i) = approx_sol(i-1) + h*((3/2)*f_val_ti-(1/2)*f_val_timinus1);
            f_val_timinus1 =  f_val_ti;
        end
    otherwise
        disp('case not implemented');
end


